//---------------------------------------------------------------------------
// TestAnisotropicDisperse
//---------------------------------------------------------------------------
#if !defined(TestAnisotropicDisperse_H)
  #define TestAnisotropicDisperse_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>
#include <cppunit/TestSuite.h>

class clGridBase;

/**
  This class tests the anisotropic dispersal behavior.  It writes out parameter
  files and uses them as input to perform setup; it then sets up the desired
  initial conditions, causes the behavior to run, then tests its output.
  <p>
  Two runs will be tested:  one in which the timestep length is 1 year and one
  in which the timestep length is 3 years.
*/
class clTestAnisotropicDisperse : public CppUnit::TestFixture {

public:
  /**
  Tests isotropic dispersal when the timestep length is 1 year.  It will cause a
  parameter file to be written and then passed to a clSimManager object.  This
  object will read it.  The disperse behavior is run and the contents of the
  seed grid examined.
  */
  void TestIsotropicDispersal1();

  /**
  Tests isotropic dispersal when the timestep length is 3 years.  It will cause
  a parameter file to be written and then passed to a clSimManager object.  This
  object will read it.  The disperse behavior is run and the contents of the
  seed grid examined.
  */
  void TestIsotropicDispersal2();

  /**
  * Tests error processing for bad input.
  */
  void TestErrorProcessing();

private:

/**
* Verifies the numbers of seeds during TestIsotropicDispersal1.
*/
  void VerifyXMLFile1(clGridBase *p_oSeedGrid);

  /**
  * Verifies the numbers of seeds during TestIsotropicDispersal2.
  */
  void VerifyXMLFile2(clGridBase *p_oSeedGrid);

  /**
    Writes a parameter file with isotropic disperse and a timestep length of 1
    year.  This contains all tree information and disperse information.
    It also has a tree map so we'll be dealing with a known list of trees.

    @return Filename written.
  */
  char* WriteXMLFile1();

  /**
    Writes a parameter file with isotropic disperse and a timestep length of 3
    years.  This contains all tree information and disperse information.
    It also has a tree map so we'll be dealing with a known list of trees.

    @return Filename written.
  */
  char* WriteXMLFile2();

  /**
  * Writes a file where an azimuth angle for anisotropic disperse is less than
  * 0.
  * @return Filename written.
  */
  char* WriteXMLErrorFile1();

  /**
  * Writes a file where an azimuth angle for anisotropic disperse is greater
  * than 2PI.
  * @return Filename written.
  */
  char* WriteXMLErrorFile2();

  /**
  * Writes a file where max search radius is less than 0.
  * @return Filename written.
  */
  char* WriteXMLErrorFile3();

  /**
  * Writes a file where max search radius is greater than 0.5 the smallest plot
  * dimension.
  * @return Filename written.
  */
  char* WriteXMLErrorFile4();
};
//---------------------------------------------------------------------------
#endif // TestAnisotropicDisperse_H
