//---------------------------------------------------------------------------
// TestRelativeGrowth
//---------------------------------------------------------------------------
#if !defined(TestRelativeGrowth_H)
  #define TestRelativeGrowth_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>
#include <cppunit/TestSuite.h>

class clSimManager;

/**
* Tests the clRelativeGrowth class.  This tests all six types of relative
* growth - unlimited relative growth, relative growth limited to constant
* radial increment, relative growth limited to constant basal area increment,
* unlimited relative growth - diameter update only, relative growth limited to
* constant radial increment - diameter update only, and relative growth limited
* to constant basal area increment - diameter update only.
*/
class clTestRelativeGrowth : public CppUnit::TestFixture {

  public:

  /**
  * Combines the testing into one function.
  */
  void TestRelativeGrowth();

  protected:

  /**
  * Tests the growth when the timestep length is one year.  WriteXMLFile1()
  * is used to set up the run.  10 trees are created of four species,
  * assigned to all six growth types and with growth levels both above and
  * below limits.  GLIs are assigned to them, and then growth is run.  Their
  * diameters, heights, and lgm values are checked to make sure they are
  * correct.
  * @param p_oSimManager Sim manager object for doing the run.
  */
  void Test1Run1YearPerTimestep(clSimManager *p_oSimManager);

  /**
  * Tests the growth when the timestep length is three years.  WriteXMLFile2()
  * is used to set up the run.  10 trees are created of three species,
  * assigned to all six growth types and with growth levels both above and
  * below limits.  GLIs are assigned to them, and then growth is run.  Their
  * diameters, heights, and lgm values are checked to make sure they are
  * correct.
  * @param p_oSimManager Sim manager object for doing the run.
  */
  void Test1Run3YearsPerTimestep(clSimManager *p_oSimManager);

  /**
  * Tests the growth when the timestep length is one year.  WriteXMLFile3()
  * is used to set up the run.  10 trees are created of four species,
  * assigned to all six growth types and with growth levels both above and
  * below limits.  There are non-1 diameter exponents.  GLIs are assigned to
  * them, and then growth is run.  Their diameters, heights, and lgm values are
  * checked to make sure they are correct.
  * @param p_oSimManager Sim manager object for doing the run.
  */
  void Test2Run1YearPerTimestep(clSimManager *p_oSimManager);

  /**
  * Tests the growth when the timestep length is three years.  WriteXMLFile4()
  * is used to set up the run.  10 trees are created of three species,
  * assigned to all six growth types and with growth levels both above and
  * below limits.  There are non-1 diameter exponents.  GLIs are assigned to
  * them, and then growth is run.  Their diameters, heights, and lgm values are
  * checked to make sure they are correct.
  * @param p_oSimManager Sim manager object for doing the run.
  */
  void Test2Run3YearsPerTimestep(clSimManager *p_oSimManager);

  /**
  * Writes a parameter file to direct testing.  Timestep length is 1 year.
  * This contains three species.
  * Behaviors:
  * <ul>
  * <li>Unlimited relative growth is applied to Species 1 saplings.</li>
  * <li>Relative growth limited to constant radial increment is applied to
  * Species 2 saplings.</li>
  * <li>Relative growth limited to basal area increment is applied to Species
  * 1 adults.</li>
  * <li>Unlimited relative growth - diameter only is applied to Species 3
  * saplings.</li>
  * <li>Relative growth limited to constant radial increment - diameter only is
  * applied to Species 4 saplings.</li>
  * <li>Relative growth limited to basal area increment - diameter only is
  * applied to Species 3 adults.</li>
  * <li>Height incrementer applied to all with diameter only behaviors.</li>
  * <li>Diameter exponents are 1.</li>
  * </ul>
  *
  * @return Filename written.
  */
  char* WriteXMLFile1();

  /**
  * Writes a parameter file to direct testing.  Timestep length is 3 years.
  * This contains three species.
  * Behaviors:
  * <ul>
  * <li>Unlimited relative growth is applied to Species 1 saplings.</li>
  * <li>Relative growth limited to constant radial increment is applied to
  * Species 2 saplings.</li>
  * <li>Relative growth limited to basal area increment is applied to Species
  * 1 adults.</li>
  * <li>Unlimited relative growth - diameter only is applied to Species 3
  * saplings.</li>
  * <li>Relative growth limited to constant radial increment - diameter only is
  * applied to Species 4 saplings.</li>
  * <li>Relative growth limited to basal area increment - diameter only is
  * applied to Species 3 adults.</li>
  * <li>Height incrementer applied to all with diameter only behaviors.</li>
  * <li>Diameter exponents are 1.</li>
  * </ul>
  *
  * @return Filename written.
  */
  char* WriteXMLFile2();

  /**
  * Writes a parameter file to direct testing.  Timestep length is 1 year.
  * This contains three species.
  * Behaviors:
  * <ul>
  * <li>Unlimited relative growth is applied to Species 1 saplings.</li>
  * <li>Relative growth limited to constant radial increment is applied to
  * Species 2 saplings.</li>
  * <li>Relative growth limited to basal area increment is applied to Species
  * 1 adults.</li>
  * <li>Unlimited relative growth - diameter only is applied to Species 3
  * saplings.</li>
  * <li>Relative growth limited to constant radial increment - diameter only is
  * applied to Species 4 saplings.</li>
  * <li>Relative growth limited to basal area increment - diameter only is
  * applied to Species 3 adults.</li>
  * <li>Height incrementer applied to all with diameter only behaviors.</li>
  * <li>Diameter exponents are non-1.</li>
  * </ul>
  *
  * @return Filename written.
  */
  char* WriteXMLFile3();

  /**
  * Writes a parameter file to direct testing.  Timestep length is 3 years.
  * This contains three species.
  * Behaviors:
  * <ul>
  * <li>Unlimited relative growth is applied to Species 1 saplings.</li>
  * <li>Relative growth limited to constant radial increment is applied to
  * Species 2 saplings.</li>
  * <li>Relative growth limited to basal area increment is applied to Species
  * 1 adults.</li>
  * <li>Unlimited relative growth - diameter only is applied to Species 3
  * saplings.</li>
  * <li>Relative growth limited to constant radial increment - diameter only is
  * applied to Species 4 saplings.</li>
  * <li>Relative growth limited to basal area increment - diameter only is
  * applied to Species 3 adults.</li>
  * <li>Height incrementer applied to all with diameter only behaviors.</li>
  * <li>Diameter exponents are non-1.</li>
  * </ul>
  *
  * @return Filename written.
  */
  char* WriteXMLFile4();

};
//---------------------------------------------------------------------------
#endif // TestRelativeGrowth_H
