//---------------------------------------------------------------------------
// TestSimpleLinearGrowth
//---------------------------------------------------------------------------
#if !defined(TestSimpleLinearGrowth_H)
  #define TestSimpleLinearGrowth_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>
#include <cppunit/TestSuite.h>

class clSimManager;

/**
* Tests the clSimpleLinearGrowth class.  This tests it in all versions - as a
* diameter-only incrementer with no height updating, as a diameter incrementer
* with auto-height updating, and as a height incrementer.
*/
class clTestSimpleLinearGrowth : public CppUnit::TestFixture {

  public:

  /**
  * Combines the testing into one behavior.
  */
  void TestSimpleLinearGrowth();

  protected:

  /**
  * Tests the growth when the timestep length is one year.  WriteXMLFile1()
  * is used to set up the run.  8 trees are created of 2 species,
  * assigned to all three growth types and with high and low GLIs.  GLIs are
  * assigned to them, and then growth is run.  Their diameters, heights, and
  * lgm values are checked to make sure they are correct.
  * @param p_oSimManager Sim manager object for doing the run.
  */
  void Test1YearPerTimestep(clSimManager *p_oSimManager);

  /**
  * Tests the growth when the timestep length is three years.  WriteXMLFile1()
  * is used to set up the run.  8 trees are created of 2 species,
  * assigned to all three growth types and with high and low GLIs.  GLIs are
  * assigned to them, and then growth is run.  Their diameters, heights, and
  * lgm values are checked to make sure they are correct.
  * @param p_oSimManager Sim manager object for doing the run.
  */
  void Test3YearsPerTimestep(clSimManager *p_oSimManager);

  /**
  * Writes a parameter file to direct testing.  Timestep length is 1 year.
  * This contains two species.
  * Behaviors:
  * <ul>
  * <li>simple linear growth diam only - applied to species 0 seedlings and species
  * 1 seedlings
  * <li>simple linear growth height only - applied to species 0 saplings and species 1 seedlings
  * <li>constant radial growth - applied to species 0 saplings
  * <li>height incrementer - applied to species 0 seedlings
  * <li> simple linear growth - applied to species 1 saplings
  * </ul>
  *
  * @return Filename written.
  */
  char* WriteXMLFile1();

  /**
  * Writes a parameter file to direct testing.  Timestep length is 3 years.
  *  This contains two species.
  * Behaviors:
  * <ul>
  * <li>simple linear growth diam only - applied to species 0 seedlings and species
  * 1 seedlings
  * <li>simple linear growth height only - applied to species 0 saplings and species 1 seedlings
  * <li>constant radial growth - applied to species 0 saplings
  * <li>height incrementer - applied to species 0 seedlings
  * <li> simple linear growth - applied to species 1 saplings
  * </ul>
  *
  * @return Filename written.
  */
  char* WriteXMLFile2();
};
//---------------------------------------------------------------------------
#endif // TestSimpleLinearGrowth_H
